#include <iostream>
#include <iterator>
#include <stdio.h>

#include "iteratorinterface.h"

#define MAXSIZE 10

class anotherStackIterator;

class anotherStack : public iteratorInterfaceBaseClass<anotherStackIterator>
{
  private:
    int intArray[MAXSIZE];
    int lastAdded;
    
  public:
    anotherStack();
    
    bool addValue(int value);
    bool addValueAt(int value, int place);
    
    int returnValueAt(int place);

    // for the iterator, return this and also NULL basically
    anotherStackIterator begin();
    anotherStackIterator end();
};

class anotherStackIterator : public iteratorInterface<int>, public std::iterator<std::forward_iterator_tag, int>
{
  private:
      int theStackRef;
      anotherStack *theStack;
      
  public:
      anotherStackIterator(anotherStack *stackP)  : theStack(stackP) { theStackRef = 0;}
      
      bool operator==(const anotherStackIterator otherValue);
      bool operator!=(const anotherStackIterator otherValue);
      int operator++();
      int operator++(int);
      
      // as from the iteratorInterface
      bool hasNext();
      int next();
      int getValue();
};

int main(int argc, char **argv) {
    anotherStack stacky;
    stacky.addValue(4);
    stacky.addValue(2);
    stacky.addValue(10);
    
    
    for (anotherStackIterator iter = stacky.begin(); iter!=stacky.end(); iter++)
    {
	printf("value %d\n", iter.getValue());
    }
    return 0;
}


anotherStack::anotherStack()
{
    lastAdded = 0;
    for (int i =0; i < MAXSIZE; i++)
      intArray[i] = -1;
}

// -1 means not on the array list
int anotherStack::returnValueAt(int place)
{
    if (place >= MAXSIZE)
      return -1;
    return intArray[place];
}

bool anotherStack::addValue(int value)
{
    if (lastAdded >= MAXSIZE)
      return false;
    else
      intArray[lastAdded] = value;
    lastAdded++;
    return true;
}

bool anotherStack::addValueAt(int value, int place)
{
    if (place >= MAXSIZE)
      return false;
    else
      intArray[place] = value;
    // no need to alter the lastAdded value, because it is more of test
    // but could check to make sure that it is not written over ?
    return true;
}



bool anotherStackIterator::operator==(const anotherStackIterator otherValue)
{
//     printf("==\n");
    if (theStackRef >=MAXSIZE)
      return false;
    return (theStack->returnValueAt(theStackRef) == otherValue.theStack->returnValueAt(otherValue.theStackRef));
}
      
      
bool anotherStackIterator::operator!=(const anotherStackIterator otherValue)
{
//     printf("!=\n");
    if (theStackRef >=MAXSIZE) 
	return false;
    if (otherValue.theStack == NULL)
	return true;
    return (theStack->returnValueAt(theStackRef) != otherValue.theStack->returnValueAt(otherValue.theStackRef));
}

int anotherStackIterator::operator++()
{
//     printf("++\n");
    if (theStackRef < MAXSIZE)
      theStackRef++;
    return theStack->returnValueAt(theStackRef);
}

int anotherStackIterator::operator++(int)
{
//     printf("++(int)\n");
    if (theStackRef < MAXSIZE)
      theStackRef++;
    return theStack->returnValueAt(theStackRef);
}

bool anotherStackIterator::hasNext()
{
    if (theStackRef >= MAXSIZE)
      return false;
    return true;
}

int anotherStackIterator::next()
{
    int tmp = theStack->returnValueAt(theStackRef);
    theStackRef++;
    return tmp;
}

int anotherStackIterator::getValue()
{
    return theStack->returnValueAt(theStackRef);
}

anotherStackIterator anotherStack::begin()
{
    return this;
}
      
anotherStackIterator anotherStack::end()
{
    return NULL;
}